@@ -36,9 +36,10 @@
 #include <src/udiskslinuxblockobject.h>
 
 #include "udiskslvm2types.h"
-#include "udiskslinuxmodulelvm2.h"
 #include "udiskslinuxmanagerlvm2.h"
 #include "udiskslvm2daemonutil.h"
+#include "udiskslvm2util.h"
+#include "udisks-lvm2-generated.h"
 #include "jobhelpers.h"
 
 /**
@@ -62,7 +63,7 @@ struct _UDisksLinuxManagerLVM2
 {
   UDisksManagerLVM2Skeleton parent_instance;
 
-  UDisksLinuxModuleLVM2 *module;
+  UDisksDaemon *daemon;
 };
 
 struct _UDisksLinuxManagerLVM2Class
@@ -73,7 +74,7 @@ struct _UDisksLinuxManagerLVM2Class
 enum
 {
   PROP_0,
-  PROP_MODULE
+  PROP_DAEMON
 };
 
 static void udisks_linux_manager_lvm2_iface_init (UDisksManagerLVM2Iface *iface);
@@ -93,8 +94,8 @@ udisks_linux_manager_lvm2_get_property (GObject    *object,
 
   switch (prop_id)
     {
-    case PROP_MODULE:
-      g_value_set_object (value, udisks_linux_manager_lvm2_get_module (manager));
+    case PROP_DAEMON:
+      g_value_set_object (value, udisks_linux_manager_lvm2_get_daemon (manager));
       break;
 
     default:
@@ -113,9 +114,10 @@ udisks_linux_manager_lvm2_set_property (GObject      *object,
 
   switch (prop_id)
     {
-    case PROP_MODULE:
-      g_assert (manager->module == NULL);
-      manager->module = g_value_dup_object (value);
+    case PROP_DAEMON:
+      g_assert (manager->daemon == NULL);
+      /* we don't take a reference to the daemon */
+      manager->daemon = g_value_get_object (value);
       break;
 
     default:
@@ -131,38 +133,26 @@ udisks_linux_manager_lvm2_init (UDisksLinuxManagerLVM2 *manager)
                                        G_DBUS_INTERFACE_SKELETON_FLAGS_HANDLE_METHOD_INVOCATIONS_IN_THREAD);
 }
 
-static void
-udisks_linux_manager_lvm2_finalize (GObject *object)
-{
-  UDisksLinuxManagerLVM2 *manager = UDISKS_LINUX_MANAGER_LVM2 (object);
-
-  g_object_unref (manager->module);
-
-  if (G_OBJECT_CLASS (udisks_linux_manager_lvm2_parent_class)->finalize)
-    G_OBJECT_CLASS (udisks_linux_manager_lvm2_parent_class)->finalize (object);
-}
-
 static void
 udisks_linux_manager_lvm2_class_init (UDisksLinuxManagerLVM2Class *klass)
 {
   GObjectClass *gobject_class;
 
   gobject_class = G_OBJECT_CLASS (klass);
-  gobject_class->finalize     = udisks_linux_manager_lvm2_finalize;
   gobject_class->set_property = udisks_linux_manager_lvm2_set_property;
   gobject_class->get_property = udisks_linux_manager_lvm2_get_property;
 
   /**
-   * UDisksLinuxManager:module:
+   * UDisksLinuxManager:daemon:
    *
-   * The #UDisksLinuxModuleLVM2 of the manager.
+   * The #UDisksDaemon for the object.
    */
   g_object_class_install_property (gobject_class,
-                                   PROP_MODULE,
-                                   g_param_spec_object ("module",
-                                                        "Module",
-                                                        "The module for the object",
-                                                        UDISKS_TYPE_LINUX_MODULE_LVM2,
+                                   PROP_DAEMON,
+                                   g_param_spec_object ("daemon",
+                                                        "Daemon",
+                                                        "The daemon for the object",
+                                                        UDISKS_TYPE_DAEMON,
                                                         G_PARAM_READABLE |
                                                         G_PARAM_WRITABLE |
                                                         G_PARAM_CONSTRUCT_ONLY |
@@ -171,57 +161,45 @@ udisks_linux_manager_lvm2_class_init (UDisksLinuxManagerLVM2Class *klass)
 
 /**
  * udisks_linux_manager_lvm2_new:
- * @module: A #UDisksLinuxModuleLVM2.
+ * @daemon: A #UDisksDaemon.
  *
  * Creates a new #UDisksLinuxManagerLVM2 instance.
  *
  * Returns: A new #UDisksLinuxManagerLVM2. Free with g_object_unref().
  */
 UDisksLinuxManagerLVM2 *
-udisks_linux_manager_lvm2_new (UDisksLinuxModuleLVM2 *module)
+udisks_linux_manager_lvm2_new (UDisksDaemon *daemon)
 {
-  g_return_val_if_fail (UDISKS_IS_MODULE (module), NULL);
+  g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), NULL);
   return UDISKS_LINUX_MANAGER_LVM2 (g_object_new (UDISKS_TYPE_LINUX_MANAGER_LVM2,
-                                                  "module", module,
+                                                  "daemon", daemon,
                                                   NULL));
 }
 
 /**
- * udisks_linux_manager_lvm2_get_module:
+ * udisks_linux_manager_lvm2_get_daemon:
  * @manager: A #UDisksLinuxManagerLVM2.
  *
- * Gets the module the @manager belongs to.
+ * Gets the daemon used by @manager.
  *
- * Returns: A #UDisksLinuxModuleLVM2. Do not free, the object is owned by @manager.
+ * Returns: A #UDisksDaemon. Do not free, the object is owned by @manager.
  */
-UDisksLinuxModuleLVM2 *
-udisks_linux_manager_lvm2_get_module (UDisksLinuxManagerLVM2 *manager)
+UDisksDaemon *
+udisks_linux_manager_lvm2_get_daemon (UDisksLinuxManagerLVM2 *manager)
 {
   g_return_val_if_fail (UDISKS_IS_LINUX_MANAGER_LVM2 (manager), NULL);
-  return manager->module;
+  return manager->daemon;
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
 
-typedef struct
-{
-  UDisksLinuxModuleLVM2 *module;
-  const gchar *name;
-} WaitForVolumeGroupObjectData;
-
 static UDisksObject *
 wait_for_volume_group_object (UDisksDaemon *daemon,
                               gpointer      user_data)
 {
-  WaitForVolumeGroupObjectData *data = user_data;
-  UDisksLinuxVolumeGroupObject *object;
+  const gchar *name = user_data;
 
-  object = udisks_linux_module_lvm2_find_volume_group_object (data->module, data->name);
-
-  if (object == NULL)
-    return NULL;
-
-  return g_object_ref (UDISKS_OBJECT (object));
+  return UDISKS_OBJECT (udisks_daemon_util_lvm2_find_volume_group_object (daemon, name));
 }
 
 static gboolean
@@ -232,7 +210,6 @@ handle_volume_group_create (UDisksManagerLVM2     *_object,
                             GVariant              *arg_options)
 {
   UDisksLinuxManagerLVM2 *manager = UDISKS_LINUX_MANAGER_LVM2(_object);
-  UDisksDaemon *daemon;
   uid_t caller_uid;
   GError *error = NULL;
   GList *blocks = NULL;
@@ -241,10 +218,9 @@ handle_volume_group_create (UDisksManagerLVM2     *_object,
   UDisksObject *group_object = NULL;
   const gchar **pvs = NULL;
   VGJobData data;
-  WaitForVolumeGroupObjectData wait_data;
 
-  daemon = udisks_module_get_daemon (UDISKS_MODULE (manager->module));
-  if (!udisks_daemon_util_get_caller_uid_sync (daemon, invocation, NULL /* GCancellable */, &caller_uid, &error))
+  error = NULL;
+  if (!udisks_daemon_util_get_caller_uid_sync (manager->daemon, invocation, NULL /* GCancellable */, &caller_uid, NULL, NULL, &error))
     {
       g_dbus_method_invocation_return_gerror (invocation, error);
       g_clear_error (&error);
@@ -252,19 +228,19 @@ handle_volume_group_create (UDisksManagerLVM2     *_object,
     }
 
   /* Policy check. */
-  UDISKS_DAEMON_CHECK_AUTHORIZATION (daemon,
+  UDISKS_DAEMON_CHECK_AUTHORIZATION (udisks_linux_manager_lvm2_get_daemon (manager),
                                      NULL,
-                                     LVM2_POLICY_ACTION_ID,
+                                     lvm2_policy_action_id,
                                      arg_options,
                                      N_("Authentication is required to create a volume group"),
                                      invocation);
 
-  if (arg_blocks == NULL || *arg_blocks == NULL)
+  if (arg_blocks == NULL)
     {
       g_dbus_method_invocation_return_error (invocation,
                                              UDISKS_ERROR,
                                              UDISKS_ERROR_FAILED,
-                                             "List of block devices is empty.");
+                                             "List of block devices is NULL!");
       goto out;
     }
 
@@ -279,7 +255,7 @@ handle_volume_group_create (UDisksManagerLVM2     *_object,
       UDisksObject *object = NULL;
       UDisksBlock *block = NULL;
 
-      object = udisks_daemon_find_object (daemon, arg_blocks[n]);
+      object = udisks_daemon_find_object (manager->daemon, arg_blocks[n]);
       if (object == NULL)
         {
           g_dbus_method_invocation_return_error (invocation,
@@ -298,15 +274,12 @@ handle_volume_group_create (UDisksManagerLVM2     *_object,
                                                  UDISKS_ERROR_FAILED,
                                                  "Object path %s for index %u is not a block device",
                                                  arg_blocks[n], n);
-          g_object_unref (object);
           goto out;
         }
 
       if (!udisks_daemon_util_lvm2_block_is_unused (block, &error))
         {
           g_dbus_method_invocation_take_error (invocation, error);
-          g_object_unref (object);
-          g_object_unref (block);
           goto out;
         }
 
@@ -315,12 +288,13 @@ handle_volume_group_create (UDisksManagerLVM2     *_object,
     }
   blocks = g_list_reverse (blocks);
 
-  pvs = (const gchar**) alloca (sizeof (gchar*) * (n + 1));
+  /* XXX: is it okay to allocate memory on stack here? */
+  pvs = (const gchar**) alloca (sizeof(gchar*) * n);
 
   /* wipe existing devices */
   for (l = blocks; l != NULL; l = l->next)
     {
-      if (!udisks_daemon_util_lvm2_wipe_block (daemon, UDISKS_BLOCK (l->data), &error))
+      if (!udisks_daemon_util_lvm2_wipe_block (manager->daemon, UDISKS_BLOCK (l->data), &error))
         {
           g_dbus_method_invocation_take_error (invocation, error);
           goto out;
@@ -334,11 +308,10 @@ handle_volume_group_create (UDisksManagerLVM2     *_object,
       PVJobData pv_data;
       pvs[n] = udisks_block_get_device (block);
       pv_data.path = pvs[n];
-      if (!udisks_daemon_launch_threaded_job_sync (daemon,
+      if (!udisks_daemon_launch_threaded_job_sync (manager->daemon,
                                                    NULL,
                                                    "lvm-pv-create",
                                                    caller_uid,
-                                                   FALSE,
                                                    pvcreate_job_func,
                                                    &pv_data,
                                                    NULL, /* user_data_free_func */
@@ -350,7 +323,6 @@ handle_volume_group_create (UDisksManagerLVM2     *_object,
                                                  UDISKS_ERROR_FAILED,
                                                  "Error creating a physical volume: %s",
                                                  error->message);
-          g_clear_error (&error);
           goto out;
         }
     }
@@ -360,11 +332,10 @@ handle_volume_group_create (UDisksManagerLVM2     *_object,
   data.vg_name = arg_name;
   data.pvs = pvs;
 
-  if (!udisks_daemon_launch_threaded_job_sync (daemon,
+  if (!udisks_daemon_launch_threaded_job_sync (manager->daemon,
                                                NULL,
                                                "lvm-vg-create",
                                                caller_uid,
-                                               FALSE,
                                                vgcreate_job_func,
                                                &data,
                                                NULL, /* user_data_free_func */
@@ -376,7 +347,6 @@ handle_volume_group_create (UDisksManagerLVM2     *_object,
                                              UDISKS_ERROR_FAILED,
                                              "Error creating volume group: %s",
                                              error->message);
-      g_clear_error (&error);
       goto out;
     }
 
@@ -386,24 +356,21 @@ handle_volume_group_create (UDisksManagerLVM2     *_object,
       UDisksObject *object_for_block;
       object_for_block = udisks_daemon_util_dup_object (block, &error);
       if (object_for_block != NULL)
-        udisks_linux_block_object_trigger_uevent_sync (UDISKS_LINUX_BLOCK_OBJECT (object_for_block),
-                                                       UDISKS_DEFAULT_WAIT_TIMEOUT);
+        udisks_linux_block_object_trigger_uevent (UDISKS_LINUX_BLOCK_OBJECT (object_for_block));
       g_object_unref (object_for_block);
     }
 
   /* ... then, sit and wait for the object to show up */
-  wait_data.module = manager->module;
-  wait_data.name = arg_name;
-  group_object = udisks_daemon_wait_for_object_sync (daemon,
+  group_object = udisks_daemon_wait_for_object_sync (manager->daemon,
                                                      wait_for_volume_group_object,
-                                                     &wait_data,
+                                                     (gpointer) arg_name,
                                                      NULL,
-                                                     UDISKS_DEFAULT_WAIT_TIMEOUT,
+                                                     10, /* timeout_seconds */
                                                      &error);
   if (group_object == NULL)
     {
       g_prefix_error (&error,
-                      "Error waiting for volume group object for '%s': ",
+                      "Error waiting for volume group object for %s",
                       arg_name);
       g_dbus_method_invocation_take_error (invocation, error);
       goto out;
@@ -412,8 +379,6 @@ handle_volume_group_create (UDisksManagerLVM2     *_object,
   udisks_manager_lvm2_complete_volume_group_create (_object,
                                                     invocation,
                                                     g_dbus_object_get_object_path (G_DBUS_OBJECT (group_object)));
-  g_object_unref (group_object);
-
  out:
   g_list_free_full (blocks, g_object_unref);
 
