@@ -23,7 +23,6 @@
 
 #include "config.h"
 #include "udiskslinuxfsinfo.h"
-#include "udisksconfigmanager.h"
 
 #define FS_EXT2      "ext2"
 #define FS_EXT3      "ext3"
@@ -71,9 +70,8 @@ const FSInfo _fs_info[] =
       NULL,
       TRUE,  /* supports_online_label_rename */
       TRUE,  /* supports_owners */
-      "mkfs.ext2 -F -L $LABEL $OPTIONS $DEVICE",
-      "mkfs.ext2 -n -F -L $LABEL $OPTIONS $DEVICE",
-      "-E nodiscard", /* option_no_discard */
+      "mkfs.ext2 -F -L $LABEL $DEVICE",
+      "mkfs.ext2 -n -F -L $LABEL $DEVICE",
     },
     {
       FS_EXT3,
@@ -81,9 +79,8 @@ const FSInfo _fs_info[] =
       NULL,
       TRUE,  /* supports_online_label_rename */
       TRUE,  /* supports_owners */
-      "mkfs.ext3 -F -L $LABEL $OPTIONS $DEVICE",
-      "mkfs.ext3 -n -F -L $LABEL $OPTIONS $DEVICE",
-      "-E nodiscard", /* option_no_discard */
+      "mkfs.ext3 -F -L $LABEL $DEVICE",
+      "mkfs.ext3 -n -F -L $LABEL $DEVICE",
     },
     {
       FS_EXT4,
@@ -91,9 +88,8 @@ const FSInfo _fs_info[] =
       NULL,
       TRUE,  /* supports_online_label_rename */
       TRUE,  /* supports_owners */
-      "mkfs.ext4 -F -L $LABEL $OPTIONS $DEVICE",
-      "mkfs.ext4 -n -F -L $LABEL $OPTIONS $DEVICE",
-      "-E nodiscard", /* option_no_discard */
+      "mkfs.ext4 -F -L $LABEL $DEVICE",
+      "mkfs.ext4 -n -F -L $LABEL $DEVICE",
     },
     {
       FS_VFAT,
@@ -103,7 +99,6 @@ const FSInfo _fs_info[] =
       FALSE, /* supports_owners */
       "mkfs.vfat -I -n $LABEL $DEVICE",
       NULL,
-      NULL, /* option_no_discard */
     },
     {
       FS_NTFS,
@@ -113,7 +108,6 @@ const FSInfo _fs_info[] =
       FALSE, /* supports_owners */
       "mkntfs -f -F -L $LABEL $DEVICE",
       "mkntfs -n -f -F -L $LABEL $DEVICE",
-      NULL, /* option_no_discard */
     },
     {
       FS_EXFAT,
@@ -123,7 +117,6 @@ const FSInfo _fs_info[] =
       FALSE, /* supports_owners */
       "mkexfatfs -n $LABEL $DEVICE",
       NULL,
-      NULL, /* option_no_discard */
     },
     {
       FS_XFS,
@@ -131,9 +124,8 @@ const FSInfo _fs_info[] =
       "xfs_admin -L -- $DEVICE",
       FALSE, /* supports_online_label_rename */
       TRUE,  /* supports_owners */
-      "mkfs.xfs -f -L $LABEL $OPTIONS $DEVICE",
-      "mkfs.xfs -N -f -L $LABEL $OPTIONS $DEVICE",
-      "-K" /* option_no_discard */
+      "mkfs.xfs -f -L $LABEL $DEVICE",
+      "mkfs.xfs -N -f -L $LABEL $DEVICE"
     },
     {
       FS_REISERFS,
@@ -143,7 +135,6 @@ const FSInfo _fs_info[] =
       TRUE,  /* supports_owners */
       "mkfs.reiserfs -q -l $LABEL $DEVICE",
       NULL,
-      NULL, /* option_no_discard */
     },
     {
       FS_NILFS2,
@@ -153,7 +144,6 @@ const FSInfo _fs_info[] =
       TRUE,  /* supports_owners */
       "mkfs.nilfs2 -L $LABEL $DEVICE",
       NULL,
-      NULL, /* option_no_discard */
     },
     {
       FS_BTRFS,
@@ -161,9 +151,8 @@ const FSInfo _fs_info[] =
       NULL,
       FALSE, /* supports_online_label_rename */
       TRUE,  /* supports_owners */
-      "mkfs.btrfs -L $LABEL $OPTIONS $DEVICE",
+      "mkfs.btrfs -L $LABEL $DEVICE",
       NULL,
-      "-K", /* option_no_discard */
     },
     {
       FS_MINIX,
@@ -173,17 +162,15 @@ const FSInfo _fs_info[] =
       FALSE, /* supports_owners */
       "mkfs.minix $DEVICE",
       NULL,
-      NULL,
     },
     {
       FS_UDF,
-      "udflabel --utf8 $DEVICE $LABEL",
+      NULL,
       NULL,
       FALSE, /* supports_online_label_rename */
-      TRUE,  /* supports_owners */
+      FALSE, /* supports_owners */
       "mkudffs --utf8 --media-type=hd --udfrev=0x201 --blocksize=$BLOCKSIZE --vid $LABEL --lvid $LABEL $DEVICE",
       NULL,
-      NULL,
     },
     {
       FS_F2FS,
@@ -193,7 +180,6 @@ const FSInfo _fs_info[] =
       TRUE,  /* supports_owners */
       "mkfs.f2fs -l $LABEL $DEVICE",
       NULL,
-      NULL,
     },
     /* swap space */
     {
@@ -272,22 +258,3 @@ get_supported_filesystems (void)
 {
   return _fs_names;
 }
-
-const gchar *_encryption_types[] =
-  {
-    UDISKS_ENCRYPTION_LUKS1,
-    UDISKS_ENCRYPTION_LUKS2,
-    NULL
-  };
-
-/**
- * get_supported_encryption_types:
- *
- * Returns: a NULL terminated list of supported encryption types. Do not free or
- * modify.
- */
-const gchar **
-get_supported_encryption_types (void)
-{
-  return _encryption_types;
-}
