@@ -317,6 +317,8 @@ handle_loop_setup (UDisksManager          *object,
   guint64 option_offset = 0;
   guint64 option_size = 0;
   uid_t caller_uid;
+  struct stat fd_statbuf;
+  gboolean fd_statbuf_valid = FALSE;
   WaitForLoopData wait_data;
 
   /* we need the uid of the caller for the loop file */
@@ -377,6 +379,12 @@ handle_loop_setup (UDisksManager          *object,
   g_variant_lookup (options, "size", "t", &option_size);
   g_variant_lookup (options, "no-part-scan", "b", &option_no_part_scan);
 
+  /* it's not a problem if fstat fails... for example, this can happen if the user
+   * passes a fd to a file on the GVfs fuse mount
+   */
+  if (fstat (fd, &fd_statbuf) == 0)
+    fd_statbuf_valid = TRUE;
+
   error = NULL;
   if (!bd_loop_setup_from_fd (fd,
                               option_offset,
@@ -416,6 +424,16 @@ handle_loop_setup (UDisksManager          *object,
                  loop_device,
                  path);
 
+  /* Update the udisks loop state file (/run/udisks2/loop) with information
+     about the new loop device created by us. We need to manually trigger
+     uevent for the device after this to update Loop interface properties. */
+  udisks_state_add_loop (udisks_daemon_get_state (manager->daemon),
+                        loop_device,
+                        path,
+                        fd_statbuf_valid ? fd_statbuf.st_dev : 0,
+                        caller_uid);
+  udisks_linux_block_object_trigger_uevent (UDISKS_LINUX_BLOCK_OBJECT (loop_object));
+
   udisks_manager_complete_loop_setup (object,
                                       invocation,
                                       NULL, /* fd_list */
@@ -867,6 +885,190 @@ handle_enable_modules (UDisksManager *object,
   return TRUE; /* returning TRUE means that we handled the method invocation */
 }
 
+static gboolean
+handle_can_format (UDisksManager         *object,
+                   GDBusMethodInvocation *invocation,
+                   const gchar           *type)
+{
+  gchar *required_utility = NULL;
+  gchar *binary_path = NULL;
+
+  if (g_strcmp0 (type, "swap") == 0)
+    required_utility = g_strdup ("mkswap");
+  else if (g_strcmp0 (type, "empty") == 0)
+    required_utility = g_strdup ("wipefs");
+  else
+    {
+      const gchar **supported_fs = get_supported_filesystems ();
+      for (gsize i = 0; supported_fs[i] != NULL; i++)
+        {
+          if (g_strcmp0 (type, supported_fs[i]) == 0)
+            {
+            required_utility = g_strconcat ("mkfs.", type, NULL);
+            break;
+            }
+        }
+    }
+
+  if (required_utility == NULL)
+    {
+      g_dbus_method_invocation_return_error (invocation,
+                                             UDISKS_ERROR,
+                                             UDISKS_ERROR_NOT_SUPPORTED,
+                                             "Creation of filesystem type %s is not supported",
+                                             type);
+      return TRUE;
+    }
+
+  binary_path = g_find_program_in_path (required_utility);
+  udisks_manager_complete_can_format (object,
+                                      invocation,
+                                      g_variant_new ("(bs)", binary_path != NULL,
+                                                     binary_path != NULL ? "" : required_utility));
+  g_free (binary_path);
+  g_free (required_utility);
+
+  return TRUE;
+}
+
+static gboolean
+handle_can_resize (UDisksManager         *object,
+                   GDBusMethodInvocation *invocation,
+                   const gchar           *type)
+{
+  GError *error = NULL;
+  gchar *required_utility = NULL;
+  BDFsResizeFlags mode;
+  gboolean ret;
+
+  ret = bd_fs_can_resize (type, &mode, &required_utility, &error);
+
+  if (error != NULL)
+    {
+      g_dbus_method_invocation_take_error (invocation, error);
+      return TRUE;
+    }
+
+  udisks_manager_complete_can_resize (object,
+                                      invocation,
+                                      g_variant_new ("(bts)", ret, (guint64) mode,
+                                                     ret ? "" : required_utility));
+
+  g_free (required_utility);
+
+  return TRUE;
+}
+
+static gboolean
+handle_can_check (UDisksManager         *object,
+                  GDBusMethodInvocation *invocation,
+                  const gchar           *type)
+{
+  GError *error = NULL;
+  gchar *required_utility = NULL;
+  gboolean ret;
+
+  ret = bd_fs_can_check (type, &required_utility, &error);
+
+  if (error != NULL)
+    {
+      g_dbus_method_invocation_take_error (invocation, error);
+      return TRUE;
+    }
+
+  udisks_manager_complete_can_check (object,
+                                     invocation,
+                                     g_variant_new ("(bs)", ret,
+                                                    ret ? "" : required_utility));
+
+  g_free (required_utility);
+
+  return TRUE;
+}
+
+static gboolean
+handle_can_repair (UDisksManager         *object,
+                   GDBusMethodInvocation *invocation,
+                   const gchar           *type)
+{
+  GError *error = NULL;
+  gchar *required_utility = NULL;
+  gboolean ret;
+
+  ret = bd_fs_can_repair (type, &required_utility, &error);
+
+  if (error != NULL)
+    {
+      g_dbus_method_invocation_take_error (invocation, error);
+      return TRUE;
+    }
+
+  udisks_manager_complete_can_repair (object,
+                                      invocation,
+                                      g_variant_new ("(bs)", ret,
+                                                     ret ? "" : required_utility));
+
+  g_free (required_utility);
+
+  return TRUE;
+}
+
+
+static GSList*
+get_block_objects (UDisksManager *manager,
+                   guint         *num_blocks)
+{
+  UDisksLinuxManager *linux_manager = UDISKS_LINUX_MANAGER (manager);
+  GDBusObjectManagerServer *object_manager = NULL;
+  GList *objects, *objects_p = NULL;
+  GSList *ret = NULL;
+
+  object_manager = udisks_daemon_get_object_manager (linux_manager->daemon);
+  objects = g_dbus_object_manager_get_objects (G_DBUS_OBJECT_MANAGER (object_manager));
+
+  for (objects_p = objects; objects_p != NULL; objects_p = objects_p->next)
+    {
+      UDisksObject *object = UDISKS_OBJECT (objects_p->data);
+      UDisksBlock *block;
+
+      block = udisks_object_get_block (object);
+      if (block != NULL)
+        {
+          ret = g_slist_prepend (ret, block);
+          (*num_blocks)++;
+        }
+    }
+
+  g_list_free_full (objects, g_object_unref);
+  ret = g_slist_reverse (ret);
+  return ret;
+}
+
+static gboolean
+handle_get_block_devices (UDisksManager         *object,
+                          GDBusMethodInvocation *invocation,
+                          GVariant              *arg_options)
+{
+  GSList *blocks, *blocks_p = NULL;
+  const gchar **block_paths = NULL;
+  guint num_blocks = 0;
+  guint i = 0;
+
+  blocks = get_block_objects (object, &num_blocks);
+  block_paths = g_new0 (const gchar *, num_blocks + 1);
+
+  for (i = 0,blocks_p = blocks; blocks_p != NULL; blocks_p = blocks_p->next, i++)
+      block_paths[i] = g_dbus_object_get_object_path (g_dbus_interface_get_object (G_DBUS_INTERFACE (blocks_p->data)));
+
+  udisks_manager_complete_get_block_devices  (object,
+                                              invocation,
+                                              block_paths);
+
+  g_slist_free_full (blocks, g_object_unref);
+
+  return TRUE;  /* returning TRUE means that we handled the method invocation */
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 static void
@@ -875,4 +1077,9 @@ manager_iface_init (UDisksManagerIface *iface)
   iface->handle_loop_setup = handle_loop_setup;
   iface->handle_mdraid_create = handle_mdraid_create;
   iface->handle_enable_modules = handle_enable_modules;
+  iface->handle_can_format = handle_can_format;
+  iface->handle_can_resize = handle_can_resize;
+  iface->handle_can_check = handle_can_check;
+  iface->handle_can_repair = handle_can_repair;
+  iface->handle_get_block_devices = handle_get_block_devices;
 }
