@@ -38,7 +38,6 @@
 #include "udiskslinuxprovider.h"
 #include "udiskslinuxmdraidobject.h"
 #include "udiskslinuxmdraid.h"
-#include "udiskslinuxmdraidhelpers.h"
 #include "udiskslinuxblockobject.h"
 #include "udisksdaemon.h"
 #include "udisksstate.h"
@@ -129,6 +128,68 @@ udisks_linux_mdraid_new (void)
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static gchar *
+read_sysfs_attr (GUdevDevice *device,
+                 const gchar *attr)
+{
+  gchar *ret = NULL;
+  gchar *path = NULL;
+  GError *error = NULL;
+
+  g_return_val_if_fail (G_UDEV_IS_DEVICE (device), NULL);
+
+  path = g_strdup_printf ("%s/%s", g_udev_device_get_sysfs_path (device), attr);
+  if (!g_file_get_contents (path, &ret, NULL /* size */, &error))
+    {
+      udisks_warning ("Error reading sysfs attr `%s': %s (%s, %d)",
+                      path, error->message, g_quark_to_string (error->domain), error->code);
+      g_clear_error (&error);
+      goto out;
+    }
+
+ out:
+  g_free (path);
+  return ret;
+}
+
+static gint
+read_sysfs_attr_as_int (GUdevDevice *device,
+                        const gchar *attr)
+{
+  gint ret = 0;
+  gchar *str = NULL;
+
+  str = read_sysfs_attr (device, attr);
+  if (str == NULL)
+    goto out;
+
+  ret = atoi (str);
+  g_free (str);
+
+ out:
+  return ret;
+}
+
+static guint64
+read_sysfs_attr_as_uint64 (GUdevDevice *device,
+                           const gchar *attr)
+{
+  guint64 ret = 0;
+  gchar *str = NULL;
+
+  str = read_sysfs_attr (device, attr);
+  if (str == NULL)
+    goto out;
+
+  ret = atoll (str);
+  g_free (str);
+
+ out:
+  return ret;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 static gboolean
 on_polling_timout (gpointer user_data)
 {
@@ -237,7 +298,6 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
   const gchar *name = NULL;
   gchar *sync_action = NULL;
   gchar *sync_completed = NULL;
-  gchar *consistency_policy = NULL;
   gchar *bitmap_location = NULL;
   guint degraded = 0;
   guint64 chunk_size = 0;
@@ -246,9 +306,9 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
   guint64 sync_remaining_time = 0;
   GVariantBuilder builder;
   UDisksDaemon *daemon = NULL;
+  gboolean has_redundancy = FALSE;
+  gboolean has_stripes = FALSE;
   UDisksBaseJob *job = NULL;
-  GError *error = NULL;
-  BDMDExamineData *raid_data = NULL;
 
   daemon = udisks_linux_mdraid_object_get_daemon (object);
 
@@ -270,17 +330,33 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
     {
       device = UDISKS_LINUX_DEVICE (member_devices->data);
       num_devices = g_udev_device_get_property_as_int (device->udev_device, "UDISKS_MD_MEMBER_DEVICES");
+      if (! num_devices)
+          num_devices = g_udev_device_get_property_as_int (device->udev_device, "STORAGED_MD_MEMBER_DEVICES");
       level = g_udev_device_get_property (device->udev_device, "UDISKS_MD_MEMBER_LEVEL");
+      if (! level)
+          level = g_udev_device_get_property (device->udev_device, "STORAGED_MD_MEMBER_LEVEL");
       uuid = g_udev_device_get_property (device->udev_device, "UDISKS_MD_MEMBER_UUID");
+      if (! uuid)
+          uuid = g_udev_device_get_property (device->udev_device, "STORAGED_MD_MEMBER_UUID");
       name = g_udev_device_get_property (device->udev_device, "UDISKS_MD_MEMBER_NAME");
+      if (! name)
+          name = g_udev_device_get_property (device->udev_device, "STORAGED_MD_MEMBER_NAME");
     }
   else
     {
       device = UDISKS_LINUX_DEVICE (raid_device);
       num_devices = g_udev_device_get_property_as_int (device->udev_device, "UDISKS_MD_DEVICES");
+      if (! num_devices)
+          num_devices = g_udev_device_get_property_as_int (device->udev_device, "STORAGED_MD_DEVICES");
       level = g_udev_device_get_property (device->udev_device, "UDISKS_MD_LEVEL");
+      if (! level)
+          level = g_udev_device_get_property (device->udev_device, "STORAGED_MD_LEVEL");
       uuid = g_udev_device_get_property (device->udev_device, "UDISKS_MD_UUID");
+      if (! uuid)
+          uuid = g_udev_device_get_property (device->udev_device, "STORAGED_MD_UUID");
       name = g_udev_device_get_property (device->udev_device, "UDISKS_MD_NAME");
+      if (! name)
+          name = g_udev_device_get_property (device->udev_device, "STORAGED_MD_NAME");
     }
 
   /* figure out size */
@@ -290,15 +366,7 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
     }
   else
     {
-      raid_data = bd_md_examine (g_udev_device_get_device_file (device->udev_device),
-                                 &error);
-      if (raid_data == NULL)
-        {
-          udisks_debug ("Failed to read array size: %s", error->message);
-          g_clear_error (&error);
-        }
-      else
-          size = raid_data->size;
+      /* TODO: need MD_ARRAY_SIZE, see https://bugs.freedesktop.org/show_bug.cgi?id=53239#c5 */
     }
 
   udisks_mdraid_set_uuid (iface, uuid);
@@ -309,26 +377,44 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
 
   udisks_mdraid_set_running (iface, raid_device != NULL);
 
+  if (g_strcmp0 (level, "raid1") == 0 ||
+      g_strcmp0 (level, "raid4") == 0 ||
+      g_strcmp0 (level, "raid5") == 0 ||
+      g_strcmp0 (level, "raid6") == 0 ||
+      g_strcmp0 (level, "raid10") == 0)
+    has_redundancy = TRUE;
+
+  if (g_strcmp0 (level, "raid0") == 0 ||
+      g_strcmp0 (level, "raid4") == 0 ||
+      g_strcmp0 (level, "raid5") == 0 ||
+      g_strcmp0 (level, "raid6") == 0 ||
+      g_strcmp0 (level, "raid10") == 0)
+    has_stripes = TRUE;
+
   if (raid_device != NULL)
     {
-      if (mdraid_has_redundancy (level))
+      if (has_redundancy)
         {
           /* Can't use GUdevDevice methods as they cache the result and these variables vary */
-          degraded = udisks_linux_device_read_sysfs_attr_as_int (raid_device, "md/degraded", NULL);
-          sync_action = udisks_linux_device_read_sysfs_attr (raid_device, "md/sync_action", NULL);
-          sync_completed = udisks_linux_device_read_sysfs_attr (raid_device, "md/sync_completed", NULL);
-          consistency_policy = udisks_linux_device_read_sysfs_attr (raid_device, "md/consistency_policy", NULL);
-          bitmap_location = udisks_linux_device_read_sysfs_attr (raid_device, "md/bitmap/location", NULL);
+          degraded = read_sysfs_attr_as_int (raid_device->udev_device, "md/degraded");
+          sync_action = read_sysfs_attr (raid_device->udev_device, "md/sync_action");
+          if (sync_action != NULL)
+            g_strstrip (sync_action);
+          sync_completed = read_sysfs_attr (raid_device->udev_device, "md/sync_completed");
+          if (sync_completed != NULL)
+            g_strstrip (sync_completed);
+          bitmap_location = read_sysfs_attr (raid_device->udev_device, "md/bitmap/location");
+          if (bitmap_location != NULL)
+            g_strstrip (bitmap_location);
         }
 
-      if (mdraid_has_stripes (level))
+      if (has_stripes)
         {
-          chunk_size = udisks_linux_device_read_sysfs_attr_as_uint64 (raid_device, "md/chunk_size", NULL);
+          chunk_size = read_sysfs_attr_as_uint64 (raid_device->udev_device, "md/chunk_size");
         }
     }
   udisks_mdraid_set_degraded (iface, degraded);
   udisks_mdraid_set_sync_action (iface, sync_action);
-  udisks_mdraid_set_consistency_policy (iface, consistency_policy);
   udisks_mdraid_set_bitmap_location (iface, bitmap_location);
   udisks_mdraid_set_chunk_size (iface, chunk_size);
 
@@ -344,7 +430,7 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
         }
 
       /* this is KiB/s (see drivers/md/md.c:sync_speed_show() */
-      sync_rate = udisks_linux_device_read_sysfs_attr_as_uint64 (raid_device, "md/sync_speed", NULL) * 1024;
+      sync_rate = read_sysfs_attr_as_uint64 (raid_device->udev_device, "md/sync_speed") * 1024;
       if (sync_rate > 0)
         {
           guint64 num_bytes_remaining = (num_sectors - completed_sectors) * 512ULL;
@@ -352,42 +438,44 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
         }
     }
 
-  if (sync_action == NULL || g_strcmp0 (sync_action, "idle") == 0)
+  if (sync_action)
     {
-      if (udisks_linux_mdraid_object_has_sync_job (object))
+      if (g_strcmp0 (sync_action, "idle") != 0)
         {
-          /* Complete the job */
-          udisks_linux_mdraid_object_complete_sync_job (object,
-                                                        TRUE,
-                                                        "Finished");
-        }
-    }
-  else
-    {
-      if (! udisks_linux_mdraid_object_has_sync_job (object))
-        {
-          /* Launch a job */
-          job = udisks_daemon_launch_simple_job (daemon,
-                                                 UDISKS_OBJECT (object),
-                                                 sync_action_to_job_id (sync_action),
-                                                 0,
-                                                 TRUE, /* no_inhibit */
-                                                 NULL  /* cancellable */);
+          if (! udisks_linux_mdraid_object_has_sync_job (object))
+            {
+              /* Launch a job */
+              job = udisks_daemon_launch_simple_job (daemon,
+                                                     UDISKS_OBJECT (object),
+                                                     sync_action_to_job_id (sync_action),
+                                                     0,
+                                                     NULL /* cancellable */);
 
-          /* Mark the job as not cancellable. It simply has to finish... */
-          udisks_job_set_cancelable (UDISKS_JOB (job), FALSE);
-          udisks_linux_mdraid_object_set_sync_job (object, job);
+              /* Mark the job as not cancellable. It simply has to finish... */
+              udisks_job_set_cancelable (UDISKS_JOB (job), FALSE);
+              udisks_linux_mdraid_object_set_sync_job (object, job);
+            }
+          else
+            job = udisks_linux_mdraid_object_get_sync_job (object);
+
+          /* Update the job's interface */
+          udisks_job_set_progress (UDISKS_JOB (job), sync_completed_val);
+          udisks_job_set_progress_valid (UDISKS_JOB (job), TRUE);
+          udisks_job_set_rate (UDISKS_JOB (job), sync_rate);
+
+          udisks_job_set_expected_end_time (UDISKS_JOB (job),
+                                            g_get_real_time () + sync_remaining_time);
         }
       else
-        job = udisks_linux_mdraid_object_get_sync_job (object);
-
-      /* Update the job's interface */
-      udisks_job_set_progress (UDISKS_JOB (job), sync_completed_val);
-      udisks_job_set_progress_valid (UDISKS_JOB (job), TRUE);
-      udisks_job_set_rate (UDISKS_JOB (job), sync_rate);
-
-      udisks_job_set_expected_end_time (UDISKS_JOB (job),
-                                        g_get_real_time () + sync_remaining_time);
+        {
+          if (udisks_linux_mdraid_object_has_sync_job (object))
+            {
+              /* Complete the job */
+              udisks_linux_mdraid_object_complete_sync_job (object,
+                                                            TRUE,
+                                                            "Finished");
+            }
+        }
     }
   udisks_mdraid_set_sync_completed (iface, sync_completed_val);
   udisks_mdraid_set_sync_rate (iface, sync_rate);
@@ -458,27 +546,30 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
                 }
 
               snprintf (buf, sizeof (buf), "md/%s/state", file_name);
-              member_state = udisks_linux_device_read_sysfs_attr (raid_device, buf, NULL);
+              member_state = read_sysfs_attr (raid_device->udev_device, buf);
               if (member_state != NULL)
                 {
+                  g_strstrip (member_state);
                   member_state_elements = g_strsplit (member_state, ",", 0);
                 }
               else
                 {
-                  /* g_variant_new() doesn't like NULL pointers for empty arrays */
+                  // g_variant_new() doesn't like NULL pointers for empty
+                  // arrays
                   member_state_elements = g_new0 (gchar*, 1);
                 }
 
               snprintf (buf, sizeof (buf), "md/%s/slot", file_name);
-              member_slot = udisks_linux_device_read_sysfs_attr (raid_device, buf, NULL);
+              member_slot = read_sysfs_attr (raid_device->udev_device, buf);
               if (member_slot != NULL)
                 {
+                  g_strstrip (member_slot);
                   if (g_strcmp0 (member_slot, "none") != 0)
                     member_slot_as_int = atoi (member_slot);
                 }
 
               snprintf (buf, sizeof (buf), "md/%s/errors", file_name);
-              member_errors = udisks_linux_device_read_sysfs_attr_as_uint64 (raid_device, buf, NULL);
+              member_errors = read_sysfs_attr_as_uint64 (raid_device->udev_device, buf);
 
               g_ptr_array_add (p,
                                g_variant_new ("(oi^asta{sv})",
@@ -503,11 +594,12 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
           /* ... and finally build (builder consumes each GVariant instance) */
           for (n = 0; n < p->len; n++)
             g_variant_builder_add_value (&builder, p->pdata[n]);
+          g_ptr_array_free (p, TRUE);
 
           g_dir_close (md_dir);
         }
       g_free (md_dir_name);
-      g_ptr_array_free (p, TRUE);
+
     }
   udisks_mdraid_set_active_devices (iface, g_variant_builder_end (&builder));
 
@@ -516,16 +608,11 @@ udisks_linux_mdraid_update (UDisksLinuxMDRaid       *mdraid,
                                                                                 uuid));
 
  out:
-  g_dbus_interface_skeleton_flush (G_DBUS_INTERFACE_SKELETON (mdraid));
-  if (raid_data)
-      bd_md_examine_data_free (raid_data);
   g_free (sync_completed);
   g_free (sync_action);
-  g_free (consistency_policy);
   g_free (bitmap_location);
   g_list_free_full (member_devices, g_object_unref);
   g_clear_object (&raid_device);
-  g_clear_error (&error);
   return ret;
 }
 
@@ -576,6 +663,7 @@ handle_start (UDisksMDRaid           *_mdraid,
   const gchar *action_id;
   const gchar *message;
   uid_t caller_uid;
+  gid_t caller_gid;
   UDisksLinuxDevice *raid_device = NULL;
   GList *member_devices = NULL;
   gchar *raid_device_file = NULL;
@@ -600,13 +688,17 @@ handle_start (UDisksMDRaid           *_mdraid,
 
   g_variant_lookup (options, "start-degraded", "b", &opt_start_degraded);
 
+  error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
+                                               &caller_gid,
+                                               NULL,
                                                &error))
     {
-      g_dbus_method_invocation_take_error (invocation, error);
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -644,7 +736,6 @@ handle_start (UDisksMDRaid           *_mdraid,
                                          UDISKS_OBJECT (object),
                                          "md-raid-start",
                                          caller_uid,
-                                         FALSE,
                                          NULL);
 
   if (job == NULL)
@@ -656,9 +747,9 @@ handle_start (UDisksMDRaid           *_mdraid,
 
   if (!bd_md_activate (NULL, NULL, udisks_mdraid_get_uuid (UDISKS_MDRAID (mdraid)), opt_start_degraded, NULL, &error))
     {
-      g_prefix_error (&error, "Error starting RAID array: ");
-      udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
+      g_prefix_error (&error, "Error starting RAID array:");
       g_dbus_method_invocation_take_error (invocation, error);
+      udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
       goto out;
     }
 
@@ -669,12 +760,12 @@ handle_start (UDisksMDRaid           *_mdraid,
                                                      wait_for_md_block_object,
                                                      object,
                                                      NULL,
-                                                     UDISKS_DEFAULT_WAIT_TIMEOUT,
+                                                     10, /* timeout_seconds */
                                                      &error);
   if (block_object == NULL)
     {
       g_prefix_error (&error,
-                      "Error waiting for MD block device after starting array: ");
+                      "Error waiting for MD block device after starting array");
       g_dbus_method_invocation_take_error (invocation, error);
       goto out;
     }
@@ -744,15 +835,18 @@ udisks_linux_mdraid_stop (UDisksMDRaid           *_mdraid,
   UDisksLinuxMDRaidObject *object;
   uid_t started_by_uid;
   uid_t caller_uid;
+  gid_t caller_gid;
   UDisksLinuxDevice *raid_device = NULL;
   UDisksBaseJob *job = NULL;
   const gchar *device_file = NULL;
-  GError *local_error = NULL;
-  gboolean ret = FALSE;
+  gboolean ret;
 
   object = udisks_daemon_util_dup_object (mdraid, error);
   if (object == NULL)
-    goto out;
+    {
+      ret = FALSE;
+      goto out;
+    }
 
   daemon = udisks_linux_mdraid_object_get_daemon (object);
   state = udisks_daemon_get_state (daemon);
@@ -761,14 +855,22 @@ udisks_linux_mdraid_stop (UDisksMDRaid           *_mdraid,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
+                                               &caller_gid,
+                                               NULL,
                                                error))
-    goto out;
+    {
+      ret = FALSE;
+      goto out;
+    }
 
   raid_device = udisks_linux_mdraid_object_get_device (object);
   if (raid_device == NULL)
     {
-      g_set_error_literal (error, UDISKS_ERROR, UDISKS_ERROR_FAILED,
-                           "RAID Array is not running");
+      g_set_error (error,
+                   UDISKS_ERROR,
+                   UDISKS_ERROR_FAILED,
+                   "RAID Array is not running");
+      ret = FALSE;
       goto out;
     }
 
@@ -798,7 +900,10 @@ udisks_linux_mdraid_stop (UDisksMDRaid           *_mdraid,
                                                                    message,
                                                                    invocation,
                                                                    error))
-        goto out;
+        {
+          ret = FALSE;
+          goto out;
+        }
     }
 
   device_file = g_udev_device_get_device_file (raid_device->udev_device);
@@ -807,21 +912,22 @@ udisks_linux_mdraid_stop (UDisksMDRaid           *_mdraid,
                                          UDISKS_OBJECT (object),
                                          "md-raid-stop",
                                          caller_uid,
-                                         FALSE,
                                          NULL);
 
   if (job == NULL)
     {
-      g_set_error_literal (error, UDISKS_ERROR, UDISKS_ERROR_FAILED,
-                           "Failed to create a job object");
+      g_dbus_method_invocation_return_error (invocation, UDISKS_ERROR, UDISKS_ERROR_FAILED,
+                                             "Failed to create a job object");
+      ret = FALSE;
       goto out;
     }
 
-  if (!bd_md_deactivate (device_file, &local_error))
+  if (!bd_md_deactivate (device_file, error))
     {
-      g_prefix_error (&local_error, "Error stopping RAID array '%s': ", device_file);
-      udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, local_error->message);
-      g_propagate_error (error, local_error);
+      g_prefix_error (error, "Error stopping RAID array %s:", device_file);
+      g_dbus_method_invocation_take_error (invocation, *error);
+      udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, (*error)->message);
+      ret = FALSE;
       goto out;
     }
 
@@ -842,9 +948,9 @@ handle_stop (UDisksMDRaid          *_mdraid,
   GError *error = NULL;
 
   if (!udisks_linux_mdraid_stop (_mdraid,
-                                 invocation,
-                                 options,
-                                 &error))
+                                   invocation,
+                                   options,
+                                   &error))
     g_dbus_method_invocation_take_error (invocation, error);
   else
     udisks_mdraid_complete_stop (_mdraid, invocation);
@@ -927,6 +1033,7 @@ handle_remove_device (UDisksMDRaid           *_mdraid,
   const gchar *message;
   uid_t started_by_uid;
   uid_t caller_uid;
+  gid_t caller_gid;
   UDisksLinuxDevice *raid_device = NULL;
   const gchar *device_file = NULL;
   const gchar *member_device_file = NULL;
@@ -950,13 +1057,17 @@ handle_remove_device (UDisksMDRaid           *_mdraid,
 
   g_variant_lookup (options, "wipe", "b", &opt_wipe);
 
+  error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
+                                               &caller_gid,
+                                               NULL,
                                                &error))
     {
-      g_dbus_method_invocation_take_error (invocation, error);
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -1029,7 +1140,6 @@ handle_remove_device (UDisksMDRaid           *_mdraid,
                                          UDISKS_OBJECT (object),
                                          "md-raid-remove-device",
                                          caller_uid,
-                                         FALSE,
                                          NULL);
 
   if (job == NULL)
@@ -1041,9 +1151,9 @@ handle_remove_device (UDisksMDRaid           *_mdraid,
 
   if (!bd_md_remove (device_file, member_device_file, set_faulty, NULL, &error))
     {
-      g_prefix_error (&error, "Error removing '%s' from RAID array '%s': ", device_file, member_device_file);
-      udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
+      g_prefix_error (&error, "Error removing %s from RAID array %s:", device_file, member_device_file);
       g_dbus_method_invocation_take_error (invocation, error);
+      udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
       goto out;
     }
 
@@ -1051,10 +1161,10 @@ handle_remove_device (UDisksMDRaid           *_mdraid,
 
   if (opt_wipe)
     {
-      if (!bd_fs_wipe (member_device_file, TRUE, TRUE, &error))
+      if (!bd_fs_wipe (member_device_file, TRUE, &error))
         {
           g_prefix_error (&error,
-                          "Error wiping '%s' after removal from RAID array '%s': ",
+                          "Error wiping  %s after removal from RAID array %s:",
                           member_device_file,
                           device_file);
           g_dbus_method_invocation_take_error (invocation, error);
@@ -1065,7 +1175,7 @@ handle_remove_device (UDisksMDRaid           *_mdraid,
   udisks_mdraid_complete_remove_device (_mdraid, invocation);
 
  out:
-  g_strfreev (member_states);
+  g_free (member_states);
   g_clear_object (&member_device_object);
   g_clear_object (&member_device);
   g_clear_object (&raid_device);
@@ -1089,6 +1199,7 @@ handle_add_device (UDisksMDRaid           *_mdraid,
   const gchar *message;
   uid_t started_by_uid;
   uid_t caller_uid;
+  gid_t caller_gid;
   UDisksLinuxDevice *raid_device = NULL;
   const gchar *device_file = NULL;
   const gchar *new_member_device_file = NULL;
@@ -1107,13 +1218,17 @@ handle_add_device (UDisksMDRaid           *_mdraid,
   daemon = udisks_linux_mdraid_object_get_daemon (object);
   state = udisks_daemon_get_state (daemon);
 
+  error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
+                                               &caller_gid,
+                                               NULL,
                                                &error))
     {
-      g_dbus_method_invocation_take_error (invocation, error);
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -1174,7 +1289,6 @@ handle_add_device (UDisksMDRaid           *_mdraid,
                                          UDISKS_OBJECT (object),
                                          "md-raid-add-device",
                                          caller_uid,
-                                         FALSE,
                                          NULL);
 
   if (job == NULL)
@@ -1186,9 +1300,9 @@ handle_add_device (UDisksMDRaid           *_mdraid,
 
   if (!bd_md_add (device_file, new_member_device_file, 0, NULL, &error))
     {
-      g_prefix_error (&error, "Error adding '%s' to RAID array '%s': ", new_member_device_file, device_file);
-      udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
+      g_prefix_error (&error, "Error adding %s to RAID array %s:", device_file, new_member_device_file);
       g_dbus_method_invocation_take_error (invocation, error);
+      udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
       goto out;
     }
 
@@ -1219,6 +1333,7 @@ handle_set_bitmap_location (UDisksMDRaid           *_mdraid,
   const gchar *message;
   uid_t started_by_uid;
   uid_t caller_uid;
+  gid_t caller_gid;
   UDisksLinuxDevice *raid_device = NULL;
   const gchar *device_file = NULL;
   GError *error = NULL;
@@ -1234,13 +1349,17 @@ handle_set_bitmap_location (UDisksMDRaid           *_mdraid,
   daemon = udisks_linux_mdraid_object_get_daemon (object);
   state = udisks_daemon_get_state (daemon);
 
+  error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
+                                               &caller_gid,
+                                               NULL,
                                                &error))
     {
-      g_dbus_method_invocation_take_error (invocation, error);
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -1291,7 +1410,6 @@ handle_set_bitmap_location (UDisksMDRaid           *_mdraid,
                                          UDISKS_OBJECT (object),
                                          "md-raid-set-bitmap",
                                          caller_uid,
-                                         FALSE,
                                          NULL);
 
   if (job == NULL)
@@ -1303,9 +1421,9 @@ handle_set_bitmap_location (UDisksMDRaid           *_mdraid,
 
   if (!bd_md_set_bitmap_location (device_file, value, &error))
     {
-      g_prefix_error (&error, "Error setting bitmap on RAID array '%s': ", device_file);
-      udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
+      g_prefix_error (&error, "Error setting bitmap on RAID array %s: ", device_file);
       g_dbus_method_invocation_take_error (invocation, error);
+      udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
       goto out;
     }
 
@@ -1335,11 +1453,11 @@ handle_request_sync_action (UDisksMDRaid           *_mdraid,
   const gchar *message;
   uid_t started_by_uid;
   uid_t caller_uid;
+  gid_t caller_gid;
   UDisksLinuxDevice *raid_device = NULL;
   GError *error = NULL;
   const gchar *device_file = NULL;
   UDisksBaseJob *job = NULL;
-  gboolean opt_no_inhibit = FALSE;
 
   object = udisks_daemon_util_dup_object (mdraid, &error);
   if (object == NULL)
@@ -1351,13 +1469,17 @@ handle_request_sync_action (UDisksMDRaid           *_mdraid,
   daemon = udisks_linux_mdraid_object_get_daemon (object);
   state = udisks_daemon_get_state (daemon);
 
+  error = NULL;
   if (!udisks_daemon_util_get_caller_uid_sync (daemon,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
+                                               &caller_gid,
+                                               NULL,
                                                &error))
     {
-      g_dbus_method_invocation_take_error (invocation, error);
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_clear_error (&error);
       goto out;
     }
 
@@ -1370,8 +1492,6 @@ handle_request_sync_action (UDisksMDRaid           *_mdraid,
       goto out;
     }
 
-  g_variant_lookup (options, "no-inhibit-lock", "b", &opt_no_inhibit);
-
   raid_device = udisks_linux_mdraid_object_get_device (object);
   if (raid_device == NULL)
     {
@@ -1412,7 +1532,6 @@ handle_request_sync_action (UDisksMDRaid           *_mdraid,
                                          UDISKS_OBJECT (object),
                                          sync_action_to_job_id (sync_action),
                                          caller_uid,
-                                         opt_no_inhibit,
                                          NULL);
 
   if (job == NULL)
@@ -1425,12 +1544,11 @@ handle_request_sync_action (UDisksMDRaid           *_mdraid,
   if (!bd_md_request_sync_action (device_file, sync_action, &error))
     {
       g_prefix_error (&error, "Error requesting '%s' action on RAID array '%s': ", sync_action, device_file);
-      udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
       g_dbus_method_invocation_take_error (invocation, error);
+      udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), FALSE, error->message);
       goto out;
     }
 
-  udisks_simple_job_complete (UDISKS_SIMPLE_JOB (job), TRUE, NULL);
   udisks_mdraid_complete_request_sync_action (_mdraid, invocation);
 
  out:
@@ -1442,30 +1560,32 @@ handle_request_sync_action (UDisksMDRaid           *_mdraid,
 /* ---------------------------------------------------------------------------------------------------- */
 
 static gboolean
-handle_delete (UDisksMDRaid           *mdraid,
-               GDBusMethodInvocation  *invocation,
-               GVariant               *options)
+udisks_linux_mdraid_delete (UDisksMDRaid           *mdraid,
+                            GDBusMethodInvocation  *invocation,
+                            GVariant               *options,
+                            GError                **error)
 {
   UDisksLinuxMDRaidObject *object;
   UDisksDaemon *daemon;
   uid_t caller_uid;
+  gid_t caller_gid;
   const gchar *message;
   const gchar *action_id;
   gboolean teardown_flag = FALSE;
   GList *member_devices = NULL;
   GList *l;
   UDisksLinuxDevice *raid_device = NULL;
-  GError *error = NULL;
+  gboolean ret;
 
   g_variant_lookup (options, "tear-down", "b", &teardown_flag);
 
   /* Delete is just stop followed by wiping of all members.
    */
 
-  object = udisks_daemon_util_dup_object (mdraid, &error);
+  object = udisks_daemon_util_dup_object (mdraid, error);
   if (object == NULL)
     {
-      g_dbus_method_invocation_take_error (invocation, error);
+      ret = FALSE;
       goto out;
     }
 
@@ -1475,9 +1595,11 @@ handle_delete (UDisksMDRaid           *mdraid,
                                                invocation,
                                                NULL /* GCancellable */,
                                                &caller_uid,
-                                               &error))
+                                               &caller_gid,
+                                               NULL,
+                                               error))
     {
-      g_dbus_method_invocation_take_error (invocation, error);
+      ret = FALSE;
       goto out;
     }
 
@@ -1489,9 +1611,9 @@ handle_delete (UDisksMDRaid           *mdraid,
                                                                options,
                                                                message,
                                                                invocation,
-                                                               &error))
+                                                               error))
     {
-      g_dbus_method_invocation_take_error (invocation, error);
+      ret = FALSE;
       goto out;
     }
 
@@ -1508,9 +1630,9 @@ handle_delete (UDisksMDRaid           *mdraid,
                                                                    options,
                                                                    message,
                                                                    invocation,
-                                                                   &error))
+                                                                   error))
         {
-          g_dbus_method_invocation_take_error (invocation, error);
+          ret = FALSE;
           goto out;
         }
 
@@ -1518,33 +1640,31 @@ handle_delete (UDisksMDRaid           *mdraid,
         {
           /* The array is running, teardown its block device.
           */
-          UDisksObject *block_object;
-          UDisksBlock *block;
-
-          block_object = udisks_daemon_find_block_by_device_file (daemon,
-                                                                  g_udev_device_get_device_file (raid_device->udev_device));
-          block = block_object ? udisks_object_peek_block (block_object) : NULL;
+          UDisksObject *block_object =
+            udisks_daemon_find_block_by_device_file (daemon,
+                                                     g_udev_device_get_device_file (raid_device->udev_device));
+          UDisksBlock *block = block_object? udisks_object_peek_block (block_object) : NULL;
           if (block &&
               !udisks_linux_block_teardown (block,
                                             invocation,
                                             options,
-                                            &error))
+                                            error))
             {
-              g_object_unref (block_object);
-              g_dbus_method_invocation_take_error (invocation, error);
+              g_clear_object (&block_object);
+              ret = FALSE;
               goto out;
             }
 
-          g_object_unref (block_object);
+          g_clear_object (&block_object);
         }
       else
         {
           /* The array is not running, remove the ChildConfiguration.
           */
           if (!udisks_linux_remove_configuration (udisks_mdraid_get_child_configuration (mdraid),
-                                                  &error))
+                                                  error))
             {
-              g_dbus_method_invocation_take_error (invocation, error);
+              ret = FALSE;
               goto out;
             }
         }
@@ -1554,9 +1674,9 @@ handle_delete (UDisksMDRaid           *mdraid,
       !udisks_linux_mdraid_stop (mdraid,
                                  invocation,
                                  options,
-                                 &error))
+                                 error))
     {
-      g_dbus_method_invocation_take_error (invocation, error);
+      ret = FALSE;
       goto out;
     }
 
@@ -1565,21 +1685,39 @@ handle_delete (UDisksMDRaid           *mdraid,
       UDisksLinuxDevice *member_device = UDISKS_LINUX_DEVICE (l->data);
       const gchar *device = g_udev_device_get_device_file (member_device->udev_device);
 
-      if (!bd_md_destroy (device, &error))
+      if (!bd_md_destroy (device, error))
         {
-          g_prefix_error (&error, "Error wiping device '%s': ", device);
-          g_dbus_method_invocation_take_error (invocation, error);
+          g_prefix_error (error, "Error wiping device %s:", device);
+          g_dbus_method_invocation_take_error (invocation, *error);
+          ret = FALSE;
           goto out;
         }
     }
 
-  udisks_mdraid_complete_delete (mdraid, invocation);
+  ret = TRUE;
 
  out:
   g_list_free_full (member_devices, g_object_unref);
   g_clear_object (&raid_device);
-  g_clear_object (&object);
-  return TRUE; /* returning TRUE means that we handled the method invocation */
+  return ret;
+}
+
+static gboolean
+handle_delete (UDisksMDRaid           *_mdraid,
+               GDBusMethodInvocation  *invocation,
+               GVariant               *options)
+{
+  GError *error = NULL;
+
+  if (!udisks_linux_mdraid_delete (_mdraid,
+                                     invocation,
+                                     options,
+                                     &error))
+    g_dbus_method_invocation_take_error (invocation, error);
+  else
+    udisks_mdraid_complete_delete (_mdraid, invocation);
+
+  return TRUE;
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
